/*
 * Project name:
     Stepper Click Board Example
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140225:
       - initial release (FJ);
 * Description:
     This is a simple demonstration of the Stepper Click Board usage. Use PD7 PD6, PD5 and PD4 buttons
     to toggle stepper motor resolution. Use PD0 and PD1 arrow buttons to initiate CW or CCW motor rotations.
     You can monitor resolution and rotations on TFT display.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz
     Ext. Modules:    Stepper Click Board - ac:Stepper_Click_Board
                      http://www.mikroe.com/click/stepper/
                      LCD 2x16 chars - ac:LCD
                      http://www.mikroe.com/store/components/displays/                     
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place DC Motor click board at the mikroBUS socket 1 on the EasyMx Pro v7 board.
     - Bipolar stepper motor connections :
          OUT2A - Blue wire
          OUT1A - Pink wire
          OUT2B - Yellow wire
          OUT1B - Orange wire
          NC    - Red wire
     - Put MOTOR PWR jumper in 5V position.
     - Pull-down PORTD/L on the EasyMx Pro v7 board.
     - Put EasyTFT in its socket, turn on backlight using the SW11.3
  */

// Bipolar Stepper motor driver connections
sbit DIR  at GPIOD_ODR.B13;
sbit STEP at GPIOA_ODR.B0;
sbit MS1 at GPIOA_ODR.B4;
sbit MS2 at GPIOC_ODR.B2;
// End Bipolar Stepper motor driver connections

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

unsigned int cnt = 0;
char lab1[10] = {0};
char lab2[17] = {0};

// Routine to execute single step
void Single_Step(){
  STEP = 0;
  Delay_ms(3);
  STEP = 1;
  Delay_ms(3);
}

void updateLabel(char *newText, int x, int y, char* lbl){
  TFT_Set_Font(&TFT_defaultFont, CL_WHITE, FO_HORIZONTAL); // Color of background
  TFT_Write_Text(lbl, x, y);
  strcpy(lbl, newText);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL); // Color of text
  TFT_Write_Text(lbl, x, y);
  delay_ms(50);
}

void DrawFrame() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("STEPPER  Click  Board  Demo", 25, 14);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7", 19, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLUE, FO_HORIZONTAL);
  TFT_Write_Text("Set resolution with PD4-PD7", 40, 150);
  TFT_Set_Font(&TFT_defaultFont, CL_BLUE, FO_HORIZONTAL);
  TFT_Write_Text("Set rotation with PD0-PD1", 40, 170);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Resolution:", 40, 80);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Rotation:", 40, 100);
  
}

void main() {
  // Set Buttons direction
  GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_LOW);
     
  // Set Bipolar Stepper motor driver direction
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_13);
  GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_0);
  GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_4);
  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_2);

  // Default start-up configuration
  DIR = 0;                       // Set clockwise direction
  MS1 = 0;                       // Set full step
  MS2 = 0;

  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);  // Initialize TFT display
  DrawFrame();

  while (1) {
    // Full Step
    if (Button(&GPIOD_IDR, 7, 10, 1)) {
      MS1 = 0;
      MS2 = 0;
      updateLabel("Full Step", 120, 80, lab1);
    }

    // 1/2 Step
    if (Button(&GPIOD_IDR, 6, 10, 1)) {
      MS1 = 1;
      MS2 = 0;
      updateLabel("1/2 Step", 120, 80, lab1);
    }

    // 1/4 Step
    if (Button(&GPIOD_IDR, 5, 10, 1)) {
      MS1 = 0;
      MS2 = 1;
      updateLabel("1/4 Step", 120, 80, lab1);
    }
    
    // 1/8 Step
    if (Button(&GPIOD_IDR, 4, 10, 1)) {
      MS1 = 1;
      MS2 = 1;
      updateLabel("1/8 Step", 120, 80, lab1);
    }

    // Rotate clockwise
    if (Button(&GPIOD_IDR, 1, 10, 1)) {
      DIR = 0;                   // Set clockwise direction
      updateLabel("Clockwise", 120, 100, lab2);
      
      // Rotate 360 degrees
      for (cnt = 0; cnt <= 2038; cnt++) {
        Single_Step();                  // Make single step
        if (Button(&GPIOD_IDR, 0, 5, 1)) {  // Change rotation direction to counterclockwise
          break;
        }
      }
      updateLabel("", 120, 100, lab2);
    }

    // Rotate counterclockwise
    if (Button(&GPIOD_IDR, 0, 10, 1)){
      DIR = 1;                   // Set counterclockwise direction
      updateLabel("Counterclockwise", 120, 100, lab2);
      
      // Rotate 360 degrees
      for (cnt = 0; cnt <= 2038; cnt++) {
        Single_Step();                  // Make single step
        if (Button(&GPIOD_IDR, 1, 5, 1))    // Change rotation direction to clockwise
          break;
      }
      updateLabel("", 120, 100, lab2);
    }
  }
}